import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.image.codec.jpeg.*;
import java.text.*;

/** Class CapturingCanvas3D, using the instructions from the Java3D 
    FAQ pages on how to capture a still image in jpeg format.

    A capture button would call a method that looks like


    public static void captureImage(CapturingCanvas3D MyCanvas3D) {
	MyCanvas3D.writeJPEG_ = true;
	MyCanvas3D.repaint();
    }


    Peter Z. Kunszt
    Johns Hopkins University
    Dept of Physics and Astronomy
    Baltimore MD
*/

public class CapturingCanvas3D extends Canvas3D  {
	public boolean rendering = false;
    public boolean writeJPEG_ = false;
    public boolean useAltCaptureName = false;
    public String captureName;
    public String capturePath;
    public String altCaptureName;
    public String fileName;
    private int postSwapCount_;
    DecimalFormat countFormat = new DecimalFormat ("#0000.#;#0000.#");
    J3DGraphics2D g2D;

    public CapturingCanvas3D(GraphicsConfiguration gc) {
    		super(gc);
    		postSwapCount_ = 0;
    		g2D = this.getGraphics2D();
    }
    
    public void preRender() {
    		rendering = true;
    }
    
    public void postRender() {
    		rendering = true;
    		//if ((writeJPEG_) & (Env.timeStampJPEGs)) {
    			g2D.setColor(Color.white);
    			g2D.drawString(Sim3DGraphics.getTimeString()+", "+Sim3DGraphics.getGluttonCtString()+ ", "+Sim3DGraphics.getMuttonCtString(),10,20);
    			//g2D.drawString(ParMSpindle_Graphics.getConcString(),30,20);
    			//g2D.drawString(ParMSpindle_Graphics.getFilCtString(),40,20);
    			//g2D.drawString(ParMSpindle_Graphics.getMonCtString(),50,20);
    			g2D.flush(false);
    		//}
    }

    public void postSwap() {
	    	if (writeJPEG_) {
	    		rendering = true;
		    int width = this.getWidth();
		    int height = this.getHeight();
		    GraphicsContext3D  ctx = getGraphicsContext3D();
		    // The raster components need all be set!
		    Raster ras = new Raster(
	                   new Point3f(-1.0f,-1.0f,-1.0f),
			   Raster.RASTER_COLOR,
			   0,0,
			   width,height,
			   new ImageComponent2D(
	                             ImageComponent.FORMAT_RGB,
				     new BufferedImage(width,height,
						       BufferedImage.TYPE_INT_RGB)),
			   null);
	
		    ctx.readRaster(ras);
	
		    // Now strip out the image info
		    BufferedImage img = ras.getImage().getImage();
	
		    // write that to disk....
		    try {
		    	if (useAltCaptureName) {
		    		fileName = capturePath +"/" + altCaptureName + ".jpg";
		    	} else {
		    		fileName = capturePath +"/" + FileOps.makeNameFromIntStep(captureName,Env.counter) + ".jpg";
		    	}
		    	System.out.print ("Writing " + fileName + " ... ");
		    	FileOutputStream out = new FileOutputStream(fileName);
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
			JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(img);
			if ((Env.jpegQuality.getFloatValue() > 1.0) | (Env.jpegQuality.getFloatValue() < 0)) {
				System.out.println ("JPEG quality out of bounds.. fix that will you.");
				param.setQuality(1.0f,false); // quality of the JPEG
			} else {
				param.setQuality(Env.jpegQuality.getFloatValue(),false); // quality of the JPEG
			}
			encoder.setJPEGEncodeParam(param);
			encoder.encode(img);
			writeJPEG_ = false;
			useAltCaptureName = false;
			out.close();
		    } catch ( IOException e ) {
			System.out.println("I/O exception!");
		    }
		    postSwapCount_++;
			System.out.println("done.");
		}
	    	rendering = false;
    }
    
    public void setAltCapture (String altName) {
    		altCaptureName = altName;
    		useAltCaptureName = true;
    }
    
    public boolean isWritingJPEG() {
    		return writeJPEG_;
    }
    
    public boolean isRendering () {
    		return rendering;
    }
    
    public boolean isBusy() {
    		if ((isWritingJPEG()) | (isRendering())) { return true; }
    		return false;
    }
    
}
