import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.Math;
import java.awt.*;
import java.io.*;
import java.text.DecimalFormat;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.JFileChooser;
import javax.swing.JFrame;

public class FileOps {
	// type definitions
	static final int NOTHING = 999;
	static final int TIME = -1;
	static final int PARM_CONCENTRATION = -2;
	static final int NUM_FILS = -3;
	static final int NUM_MONS = -4;
	static final int NUM_THINGS = 0;
	static final int FILAMENT = 1;
	static final int PLASMID = 2;
	static final int BUG = 3;
	
	static String sepString = ";";
	static String divStr = ":";
	static String endStr = ";";
	static String idDivStr = ":";
	static String commentStr = "//";
	// file, file writer, and print writer for program errors to file
	static File errorsOutFile;
	static FileOutputStream errorsOutFOS;
	static PrintStream errorsOutPS;	
	// file, file writer, and print writer for program messages to file
	static File messagesOutFile;
	static FileWriter messagesOutFW;
	static PrintWriter messagesOutPW;	
	// file, file writer, and print writer for information file
	static File outFile;
	static FileWriter outFW;
	static PrintWriter outPW;
	// files and writers for filament lengths file
	static File filLengthFile;
	static FileWriter filLengthFW;
	static PrintWriter filLengthPW;
	// flags for file setup
	static boolean outFileMade = false;
	static boolean filLengthFileMade = false;
	
	// file id formating
	static DecimalFormat fileIdFormat = new DecimalFormat ("#000000000.#;#000000000.#");


	public FileOps () {
	}
	

	// A class to filter out .java files
	class SourceFileFilter implements FilenameFilter {

    /**
    * Select only .java files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".java" );
      }
    }
	
	//	 A class to filter out .param files
	class ParamFileFilter implements FilenameFilter {

    /**
    * Select only .param files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".param" );
      }
    }
	
	// A class to filter out .qk files
	static class QKFileFilter implements FilenameFilter {

    /**
    * Select only .qk files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".qk" );
      }
    }
    
    public void copyFile(File in, File out) throws Exception {
	    FileInputStream fis  = new FileInputStream(in);
	    FileOutputStream fos = new FileOutputStream(out);
	    byte[] buf = new byte[1024];
	    int i = 0;
	    while((i=fis.read(buf))!=-1) {
	      fos.write(buf, 0, i);
	    }
	    fis.close();
	    fos.close();
	}
    
    public static String [] getQKFileList (File dir) {
    		return dir.list(new QKFileFilter());
    }
    
	public static String getDirectoryToSave (String dialogTitle, Component parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		saveTo.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            if (!file.exists()) { file.mkdir(); }
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getFileToSave (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getDirectoryToLoad (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser loadFrom = new JFileChooser();
		loadFrom.setDialogTitle(dialogTitle);
		loadFrom.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = loadFrom.showOpenDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = loadFrom.getSelectedFile();
            fileName = file.getAbsolutePath();
            //talkln (dialogTitle + fileName);
        } else {
            talkln ("Load cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getFileToLoad (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser loadFrom = new JFileChooser();
		loadFrom.setDialogTitle(dialogTitle);
		loadFrom.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int fileQuery = loadFrom.showOpenDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = loadFrom.getSelectedFile();
            fileName = file.getAbsolutePath();
            //talkln (dialogTitle + fileName);
        } else {
            talkln ("Load cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String makeNameFromIntStep (String base, int integrationStep) {
		return (base + String.valueOf(fileIdFormat.format(integrationStep)));
	}
	
	public void setUpFiles () {
		mkErrorFile();
		mkMessageFile(); 
		//mkSourceFolder();
		mkOutFile();
	}
	
	public void mkErrorFile () {
		try {
			errorsOutFile = new File (Env.outFolderPath + File.separator + Env.outFileName + "Errors");
			errorsOutFOS = new FileOutputStream(errorsOutFile);
			errorsOutPS = new PrintStream(errorsOutFOS);
			System.setErr(errorsOutPS);
			//Date timeNow = new Date();
			//errorsOutPW.println("This is the errors file for " + Env.outFileName + " run on " + timeNow.toString());
		} catch (IOException ioe) { talkln ("An error creating errorsOut stuff"); }
	}
		
	public void mkMessageFile () {
		try {
			messagesOutFile = new File (Env.outFolderPath + File.separator + Env.outFileName + "Messages");
			messagesOutFW = new FileWriter(messagesOutFile);
			messagesOutPW = new PrintWriter(messagesOutFW, true);
			Date timeNow = new Date();
			messagesOutPW.println("This is the message file for " + Env.outFileName + " run on " + timeNow.toString());
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public void mkEnvironmentFile () {
		File envFile = new File ("Env.java");
		if ( ! envFile.exists() || ! envFile.canRead() ) {
			talkln ("Can't find 'Env.java'... you'd better store those values manually");
		} else {
			try {
				FileReader fileR = new FileReader(envFile);
				BufferedReader in = new BufferedReader(fileR);
				File envOut = new File(Env.outFolderPath + File.separator + Env.outFileName + "Environment");
				FileWriter envOutFW = new FileWriter(envOut);
				PrintWriter envOutPW = new PrintWriter(envOutFW, true);
				Date timeNow = new Date();
				envOutPW.println("This is the environment file for " + Env.outFileName + " run on " + timeNow.toString());
				String nextLine = in.readLine();
				while (nextLine != null) {
					envOutPW.println(nextLine);
					nextLine = in.readLine();
				}
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	public void mkSourceFolder () {
		try {
			File curDir = new File (".");
			System.out.println ("Directory for source files is: " + curDir.getCanonicalPath());
			String [] srcFiles = curDir.list(new SourceFileFilter());
			for (int i=0;i<srcFiles.length;i++) {
				File curSrcFile = new File (srcFiles[i]);
				File curOutSrcFile = new File (Env.srcFilePath + File.separator + curSrcFile.getName());
				copyFile(curSrcFile,curOutSrcFile);
			}
		} catch (Exception e) { talkln ("An error trying to copy source files"); }
	}
	
	public static void mkOutFile () {
		try {
			outFile = new File (Env.outFolderPath + File.separator + Env.outFileName + "Out");
			outFW = new FileWriter(outFile);
			outPW = new PrintWriter(outFW,true);
			Date timeNow = new Date();
			outPW.println("This is the out file for " + Env.outFileName + " run on " + timeNow.toString());
			outPW.print("time;");
			String iStr;
			// this next part is for the plasmid positions header
			for (int i=0;i<SphereThing.sphereCt;i++) {
				iStr = String.valueOf(i);
				outPW.print("plasmid" + iStr + ".x;plasmid" + iStr + ".y;plasmid" + iStr + ".z;");
			}
			outPW.print("ParM concentration;");
			outPW.print("fil length ave;");
			outPW.print("fil length stdev;");
			outPW.println("");
			outFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating out file stuff"); }
	}
	
	public static void mkFilLengthFile () {
		try {
			filLengthFile = new File (Env.outFolderPath + File.separator + Env.outFileName + "Filaments");
			filLengthFW = new FileWriter(filLengthFile);
			filLengthPW = new PrintWriter(filLengthFW,true);
			Date timeNow = new Date();
			filLengthPW.println("This is the filament lengths file for " + Env.outFileName + " run on " + timeNow.toString());
			filLengthPW.print("time;");
			filLengthPW.println("");
			filLengthFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating filament length file stuff"); }
	}
	
	public static void mkHistogramFile (String fileName, double[] intervalVals, int [] ctVals) {
		try { 
			File histoFile = new File (Env.outFolderPath + File.separator + Env.outFileName + fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			histoPW.println("interval; count");
			for (int i=0; i<ctVals.length; i++) {
				histoPW.println(String.valueOf(intervalVals[i]) + ";" + String.valueOf(ctVals[i]));
			}
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	
	private static void printDivider (PrintWriter pw) {
		pw.print(sepString);
	}
	
	private static void printValue (int val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printValue (double val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printPt3D (Pt3D pt, PrintWriter pw) {
		pw.print(String.valueOf(pt.x));
		printDivider(pw);
		pw.print(String.valueOf(pt.y));
		printDivider(pw);
		pw.print(String.valueOf(pt.z));
		printDivider(pw);
	}
	
	public static void remoteParamConfigSave () {
		File configFile = new File (Env.outFolderPath + File.separator + Env.outFileName + "Params");
		writeParamConfig (configFile);
	}
	
	public static void writeParamConfig (File paramOutFile) {
		talk ("Saving parameter values to " + paramOutFile.getAbsolutePath() + "...");
		try {
			FileWriter paramOutFW = new FileWriter(paramOutFile);
			PrintWriter paramOutPW = new PrintWriter(paramOutFW, true);
			Date timeNow = new Date();
			paramOutPW.println("// This is a parameter configuration file created on " + timeNow.toString());
			Parameter curParam;
			String firstStr;
			for (int i=0;i<Parameter.paramCt;i++) {
				curParam = Parameter.theParams[i];
				firstStr = curParam.label + divStr + String.valueOf(curParam.isActive()) + divStr + String.valueOf(curParam.getValue()) + endStr;
				paramOutPW.print(firstStr);
				int spacesToAdd = 40-firstStr.length();
				for (int j=0;j<spacesToAdd;j++) { paramOutPW.print(" "); }
				paramOutPW.println(commentStr + curParam.parameterName);
			}
			talkln ("done.");
		} catch (IOException ioe) { talkln ("An error trying to write the parameter config file"); }
	}

	public static void loadParamConfig (File paramFile) {
		talkln ("Loading parameter values from " + paramFile.getAbsolutePath() + "...");
		int misunderstandings = 0;
		if ( ! paramFile.exists() || ! paramFile.canRead() ) {
			talkln ("Can't find or access " + paramFile.getAbsolutePath() + " ... check the file path and name");
		} else {
			try {
				FileReader fileR = new FileReader(paramFile);
				BufferedReader in = new BufferedReader(fileR);
				String curLine = in.readLine();
				String curLabel,curValueStr,curActiveStr;
				boolean curActive = true;
				double curValue = 0;
				int div1Ind,div2Ind,endInd;
				while (curLine != null) {
					if (!curLine.startsWith(commentStr)) {
						// parse label and value strings
						div1Ind = curLine.indexOf(divStr);
						div2Ind = curLine.indexOf(divStr,div1Ind+1);
						endInd = curLine.indexOf(endStr);
						curLabel = curLine.substring(0,div1Ind).trim();
						curActiveStr = curLine.substring(div1Ind+1,div2Ind).trim();
						curValueStr = curLine.substring(div2Ind+1,endInd);
						// make value string a double
						try {
							Double valD = Double.valueOf(curValueStr);
							curValue = valD.doubleValue();
						} catch (java.lang.NumberFormatException nfe) {
							System.out.println("Invalid parameter value, " + curValueStr + " loaded from " + paramFile.getAbsolutePath() + "... exiting");
							System.exit(0);
						}
						// set boolean
						if (curActiveStr.equals("false")) { curActive = false; } else { curActive = true; }
						// find the matching parameter label from static list of parameters
						boolean matchFound = false;
						for (int i=0;i<Parameter.paramCt;i++) {
							if (Parameter.theParams[i].label.equals(curLabel)) {
								Parameter.theParams[i].setValue(curValue);
								Parameter.theParams[i].setActive(curActive);
								matchFound = true;
								talkln ("     Parameter assigned: " + curLabel + " set to " + String.valueOf(Parameter.theParams[i].getValue()));
							}
							if (matchFound) { break; }
						}
						if (!matchFound) { 
							talkln ("     No match found for parameter label " + curLabel + " ... you best check that out!"); 
							misunderstandings++;
						}
					}
					// next line
					curLine = in.readLine();
				}
			} catch (IOException ioe) { talkln ("An error trying to load param file"); }
		}
		ParamGui.syncAllToParameters();
		talkln ("Done... there were " + String.valueOf(misunderstandings) + " misunderstandings.");
		talk ("Restarting simulation... ");
		Sim3DGraphics.restartRun();
		if (Env.remote) { 
			Sim3DGraphics.setRunning(); 
		}
		talkln("done.");
	}
	
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.print(info);
	}
}
