/**
 * Glutton.java
 *
 * 
 */



import java.awt.*;
import com.sun.j3d.utils.geometry.Sphere;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Glutton extends SphereThing {
	static Glutton [] theGluttons = new Glutton [Env.maxThings];	// array holding all plasmids
	static int gluttonCt = 0;			//
	static Object gluttonSyncO = new Object();		// object for synchronized addition/removal of gluttons
	int gluttonNumber;
	Color gluttonColor = Color.blue;	// color for drawing
	
	public Glutton (Pt3D initCoord) {
		super(initCoord,Env.gluttonRadius.getValue());
		addGlutton(this);
		ambientC = new Color3f(gluttonColor);
		diffuseC = new Color3f(gluttonColor);
	}
	
	public void step () {
		super.step();
		shrink();
	}
	
	public void grow (double eatenRadius) {
		double oldRadius = radius;
		double radiiCubedSum = Math.pow(eatenRadius,3) + Math.pow(radius,3);
		radius = Math.pow(radiiCubedSum,1.0/3.0);
		if (radius > Env.gluttonBurstSize.getValue()) { removeMe = true; }
		calculateProperties();
		//graphics
		t3d.setScale((radius/oldRadius)*t3d.getScale());
		g3d.setTransform(t3d);

	}
	
	public void shrink () {
		double fracLeft = 1-Env.gluttonMetabolicRate.getValue()*Env.deltaT.getValue();
		radius = fracLeft*radius;
		if (radius < Env.gluttonWitherSize.getValue()) { removeMe = true; }
		calculateProperties();
		//graphics
		t3d.setScale(fracLeft*t3d.getScale());
		g3d.setTransform(t3d);
	}
	
	public static void makeInitialGluttons () {
		for (int i=0;i<Env.initialGluttons.getValue();i++) {
			makeRandomGlutton();
		}
	}
		
	public static void makeRandomGlutton () {
		Pt3D randomLoc = Thing.theSphere.rdmPtInsideBug(Env.gluttonRadius.getValue());
		new Glutton (randomLoc);
	}
	
	public static void addGlutton (Glutton newGlutton) {
		synchronized (gluttonSyncO) {
			theGluttons[gluttonCt] = newGlutton;
			theGluttons[gluttonCt].gluttonNumber = gluttonCt;
			gluttonCt++;
		}
	}
	
	public void swapOut () {
		super.swapOut();
		System.out.println("gluttonNumber = " + String.valueOf(gluttonNumber));
		synchronized (gluttonSyncO) {
			int swapId = gluttonNumber;
			theGluttons[swapId] = theGluttons[gluttonCt-1];
			theGluttons[swapId].gluttonNumber = swapId;
			theGluttons[gluttonCt-1] = null;
			gluttonCt--;
		}
	}
	
	public void nullify () {
		super.nullify();
		gluttonColor = null;
	}
	
	public static void checkMuttonCollision () {
		Glutton curGlutt;
		Mutton curMutt;
		for (int i=0; i<gluttonCt; i++) {
			curGlutt = theGluttons[i];
			for (int p=0;p<Mutton.muttonCt;p++) {
				curMutt = Mutton.theMuttons[p];
				double pDist = Pt3D.ptDist(curGlutt.coord, curMutt.coord);
				if (pDist<curMutt.getRadius()+curGlutt.getRadius()) {
					curMutt.removeMe = true;
					curGlutt.grow(curMutt.radius);
				}
			}
		}
	}
	


}
