/*
 * InitControlFrame... vary initial conditions for the run
*/

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.media.j3d.*;

public class GraphicsControl extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	static Sim3DGraphics parentSim;
	static JPanel titlePanel,initsPanel,applyPanel;
	static JCheckBox allowResizeBox;
	static BareButton cancelB,applyB;
	static ParamGui fWidthGui, fHeightGui;
	static ParamGui [] inits = new ParamGui[10];
	static int initCt = 0;
	static boolean frameSizeEdited = false;
	
	public GraphicsControl (Sim3DGraphics parentSim) {
		this.parentSim = parentSim;
		
		makeInitsPanel();
		makeApplyPanel();
		
		if (!Env.remote) {
			addKeyListener(this);
			addComponentListener(this);
			addMouseListener(this);
			addMouseMotionListener(this);
			addWindowListener(this);
			
			initialize();
			this.setTitle("Graphics & File Writing");
			Dimension prefDim = getPreferredSize();
			prefDim.height += (initCt+1)*10;
			setSize(prefDim);
		}
		
	}
	
	public void initialize () {
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		//getContentPane().add (titlePanel, BorderLayout.NORTH);
		getContentPane().add (initsPanel, BorderLayout.CENTER);
		getContentPane().add (applyPanel, BorderLayout.SOUTH);
	}
	
	public void setInfoFields () {
		for (int i=0;i<initCt;i++) {
			inits[i].syncToParameter();
		}
	}
	
	public void updateInfoFields () {
		for (int i=0;i<initCt;i++) {
			if (!inits[i].isInEdit()) {
				inits[i].syncToParameter();
			} else {
				if (inits[i] == fWidthGui) { frameSizeEdited = true; }
				if (inits[i] == fHeightGui) { frameSizeEdited = true; }
				editsOn();
			}
		}
	}
	
	public void refreshAll () {
		for (int i=0;i<initCt;i++) { inits[i].syncToParameter(); }
	}
	
	public void revertAll () {
		for (int i=0;i<initCt;i++) { inits[i].setToDefaultValue(); }
	}

	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Apply")) {
			for (int i=0;i<initCt;i++) { inits[i].setValueFromGui(); }
			refreshAll();
			if (frameSizeEdited) {
				parentSim.setCanvasSize(Env.frameWidth.getIntValue(),Env.frameHeight.getIntValue());
				frameSizeEdited = false;
			}
			editsOff();
		}
		
		if (act.equals("Revert")) {
			revertAll();
			parentSim.setCanvasSize(Env.frameWidth.getIntValue(),Env.frameHeight.getIntValue());
			editsOff();
		}
		
		if (act.equals("Cancel")) {
			refreshAll();
			editsOff();
		}
		
		if (act.equals("AllowDragResize")) {
			Env.allowDragResize = allowResizeBox.isSelected();
			Sim3D.simGraphics.setDragResize(Env.allowDragResize);
		}
		
	}
	
	public void stateChanged(ChangeEvent e) {
	    JSlider source = (JSlider)e.getSource();
	}
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	public void makeTitlePanel () {
		titlePanel = new JPanel();
		titlePanel.setBackground(Env.controlBackColor);
		JLabel titleLabel = new JLabel("Graphics");
		titleLabel.setForeground(Env.controlForeColor);
		titlePanel.add(titleLabel);
	}
	
	private void addInit (ParamGui newInit) {
		inits[initCt] = newInit;
		inits[initCt].addToPanel(initsPanel);
		initCt++;
	}
	
	public void makeInitsPanel() {
		initsPanel = new JPanel();
		initsPanel.setBackground(Env.controlBackColor);
		initsPanel.setLayout(new GridLayout(10,3));
		
		InertGuiHead frameSizeHead = new InertGuiHead(" Frame Size","","");
		frameSizeHead.addToPanel(initsPanel);
		
		fWidthGui = new ParamGui(Env.frameWidth);
		addInit(fWidthGui);
		fHeightGui = new ParamGui(Env.frameHeight);
		addInit(fHeightGui);
		
		allowResizeBox = new JCheckBox(" Allow resize by drag (careful Macs)",Env.allowDragResize);
		allowResizeBox.setActionCommand("AllowDragResize");
		allowResizeBox.addActionListener(this);
		allowResizeBox.setBackground(Env.controlBackColor);
		allowResizeBox.setForeground(Env.controlForeColor);
		allowResizeBox.setFont(Env.controlFont);
		initsPanel.add(allowResizeBox);
		initsPanel.add(new JLabel(""));
		initsPanel.add(new JLabel(""));
		
		
		addBlankHead();
		
		InertGuiHead drawIntervalHead = new InertGuiHead(" Drawing & File Writing","","");
		drawIntervalHead.addToPanel(initsPanel);
		
		addInit(new ParamGui(Env.drawInterval));
		addInit(new ParamGui(Env.toFileInterval));
		addInit(new ParamGui(Env.jpegQuality));
		addInit(new ParamGui(Env.remoteReportInterval));

	}
	
	public void addBlankHead() {
		InertGuiHead.addToPanel(initsPanel, new InertGuiHead("","",""));
	}
	
	public void makeApplyPanel () {
		applyPanel = new JPanel();
		applyPanel.setBackground(Env.controlBackColor);
	
		cancelB = new BareButton ("Cancel");
		cancelB.setActionCommand("Cancel");
		cancelB.setToolTipText("Cancel these changes");
		cancelB.addActionListener(this);
		cancelB.setEnabled(false);
		
		BareButton revertB = new BareButton ("Revert");
		revertB.setActionCommand("Revert");
		revertB.setToolTipText("Revert to default values");
		revertB.addActionListener(this);
		revertB.setEnabled(true);
		
		applyB = new BareButton("Apply");
		applyB.setActionCommand("Apply");
		applyB.setToolTipText("Apply Changes");
		applyB.addActionListener(this);
		applyB.setEnabled(false);
		
		applyPanel.setLayout(new FlowLayout(FlowLayout.CENTER,10,10));
		applyPanel.add(cancelB);
		applyPanel.add(revertB);
		applyPanel.add(applyB);
		
	}
	
	public void editsOn () {
		cancelB.setEnabled(true);
		applyB.setEnabled(true);
	}
	
	public void editsOff () {
		cancelB.setEnabled(false);
		applyB.setEnabled(false);
	}
}
