
public class Parameter {
	static Parameter [] theParams = new Parameter[100];
	static int paramCt = 0;
	static final int DOUBLE = 0;
	static final int INT = 1;
	static final int CHANGE_VALUE = 11;
	static final int MECHANISM_ON = 12;
	int type;
	int activeType;
	String label;			// permanent reference for this parameter... changing value can affect the ability to load older config files
	String parameterName;
	String units;
	double defaultValue;
	double curValue;
	double lastValue;
	boolean positiveValue = true;		// by default.. only positive values for parameters
	private boolean active = true;
	boolean dependent = false;
	Parameter dependsOn;

	
	public Parameter (String label, String parameterName, double defaultValue, String units) {
		this.label = label;
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.type = Parameter.DOUBLE;
		this.activeType = Parameter.CHANGE_VALUE;
		addParameter(this);
	}
	
	public Parameter (String label, String parameterName, double defaultValue, String units,int type) {
		this.label = label;
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.type = type;
		this.activeType = Parameter.CHANGE_VALUE;
		addParameter(this);
		//System.out.println(parameterName + " curValue is " + String.valueOf(curValue));
	}
	
	public Parameter (String label, String parameterName, double defaultValue, String units,int type,int activeType) {
		this.label = label;
		this.parameterName = parameterName;
		this.defaultValue = defaultValue;
		curValue = defaultValue;
		setLastValue(curValue);
		this.units = units;
		this.type = type;
		this.activeType = activeType;
		addParameter(this);
	}
	
	public void setValue (double newValue) {
		if (!dependent) {
			curValue = newValue;
		} else {
			curValue = dependsOn.getValue();
			active = dependsOn.isActive();
		}
	}
	

	public double getValue () {
		if (!dependent) {
			return curValue;
		} else {
			return dependsOn.getValue();
		}
	}
	
	public float getFloatValue () {
		return (float) getValue();
	}
	
	public int getIntValue () {
		return (int) getValue();
	}
	
	public String getStringValue () {
		return String.valueOf(getValue());
	}
	
	public void addToValue (double addVal) {
		if (active) {
			curValue += addVal;
			if ((positiveValue) & (curValue < 0)) {
				curValue = 0;
			}
		}
	}
	
	public void resetToDefault () {
		curValue = defaultValue;
	}
	
	public void resetToLastValue () {
		curValue = lastValue;
	}
	
	public void setLastValue (double newLastVal) {
		lastValue = newLastVal;
	}
	
	public double getLastValue () {
		return lastValue;
	}
	
	public void setName (String name) {
		parameterName = name;
	}
	
	public String getName () {
		return parameterName;
	}
	
	public String getUnits () {
		return units;
	}
	
	public void setPositiveValueOnly (boolean pos) {
		positiveValue = pos;
	}
	
	public void setActive (boolean active) {
		this.active = active;
	}
	
	public boolean isActive () {
		if (!dependent) {
			return active;
		} else {
			return dependsOn.isActive();
		}
	}
	
	public void setDependence (Parameter dependsOn) {
		dependent = true;
		this.dependsOn = dependsOn;
	}
	
	public void setDependence () {
		dependent = false;
		dependsOn = null;
	}
	
	public static void addParameter (Parameter addP) {
		theParams[paramCt] = addP;
		paramCt++;
	}
	
	
			
}
