
import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.vecmath.*;

import java.io.File;
import java.text.DecimalFormat;

import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;

public class Sim3DGraphics extends JFrame
	implements WindowListener, CheckboxMenuListener, ActionListener, ItemListener {

	//  Should the scene be compiled?
	private boolean shouldCompile = true;

	//  GUI objects for our subclasses
	protected Frame    theFrame    = null;
	protected MenuBar  theMenuBar  = null;
	protected static CapturingCanvas3D theCanvas = null;
	protected static JPanel topPanel = null;
	protected static JLabel timeLabel = null;
	protected static JLabel concLabel = null;
	protected static JLabel filCtLabel = null;
	protected static JLabel monCtLabel = null;
	protected TransformGroup theViewTransform = null;
	protected TransformGroup theSceneTransform = null;

	//  Private GUI objects and state
	public DirectionalLight headlight = null;
	//private ExamineViewerBehavior examineBehavior = null;

	private Background background = null;
	private static Group scene = null;
	private static BranchGroup bugState = null;
	
	
	// for menus
	private static boolean headlightOnOff = true;
	private static CheckboxMenuItem headlightMenuItem = null;
	private CheckboxMenuItem paintOnMenuItem = null;
	private static CheckboxMenuItem writeFileMenuItem = null;
	private static CheckboxMenuItem pausedMenuItem = null;
	private static CheckboxMenuItem runMenuItem = null;
	private static CheckboxMenuItem rotateMenuItem = null;
	private static CheckboxMenuItem sphereMotionOffMenuItem = null;
	private CheckboxMenuItem[] appearanceMenu;
	
	// control frames
	public static ParamControl ratesControl;
	public static InitControl initControl;
	public static GraphicsControl graphicsControl;
	
	
	// text formats
	static DecimalFormat timeFormat = new DecimalFormat ("0.0000");
	static DecimalFormat concFormat = new DecimalFormat ("0.0000");
	
	public Sim3DGraphics( )
	{
		ratesControl = new ParamControl(this);
		initControl = new InitControl(this);
		graphicsControl = new GraphicsControl(this);
	}


	 //  Initializes the application when invoked as an applet.
	public void init( ) {
		this.initialize();
		this.buildUniverse( );
		this.showFrame( );
	}


	/**
	 *  Initializes the Example by parsing command-line arguments,
	 *  building an AWT Frame, constructing a menubar, and creating
	 *  the 3D canvas.
	 *
	 *  @param   args    a String array of command-line arguments
	 */
	protected void initialize() {

		// Build the frame
		theFrame = new Frame( );
		theFrame.setBackground(Env.universeColor);
		theFrame.setSize( Env.frameWidth.getIntValue(), Env.frameHeight.getIntValue() );
		theFrame.setTitle( "" );
		theFrame.setLayout( new BorderLayout( ) );

		// Set up a close behavior
		theFrame.addWindowListener( this );
		
		// Create a canvas
		theCanvas = new CapturingCanvas3D(SimpleUniverse.getPreferredConfiguration());
		theCanvas.setSize( Env.frameWidth.getIntValue(), Env.frameHeight.getIntValue() );
		theFrame.setResizable(Env.allowDragResize);
		theFrame.add( "Center", theCanvas );
		
		// Pack
		theFrame.pack( );
		theFrame.validate( );
		
		theFrame.setTitle( "Your 3D simulation... change title" );
		makeMenuBar();
	}



	//--------------------------------------------------------------
	//  SCENE CONTENT
	//--------------------------------------------------------------

	/**
	 *  Builds the 3D universe by constructing a virtual universe
	 *  (via SimpleUniverse), a view platform (via SimpleUniverse), and
	 *  a view (via SimpleUniverse).  A headlight is added and a
	 *  set of behaviors initialized to handle navigation types.
	 */
	protected void buildUniverse( ) {
		SimpleUniverse universe = new SimpleUniverse(theCanvas);
		
		ViewingPlatform viewingPlatform = universe.getViewingPlatform( );
		OrbitBehavior orbit = new OrbitBehavior(theCanvas,OrbitBehavior.REVERSE_ALL);
		BoundingSphere bounds = new BoundingSphere(new Point3d( 0.0, 0.0, 0.0 ), 100000.0 );
		orbit.setSchedulingBounds(bounds);
		viewingPlatform.setViewPlatformBehavior(orbit);
		theViewTransform = viewingPlatform.getViewPlatformTransform( );


		//
		//  Create a "headlight" as a forward-facing directional light.
		//  Set the light's bounds to huge.  Since we want the light
		//  on the viewer's "head", we need the light within the
		//  TransformGroup containing the ViewPlatform.  The
		//  ViewingPlatform class creates a handy hook to do this
		//  called "platform geometry".  The PlatformGeometry class is
		//  subclassed off of BranchGroup, and is intended to contain
		//  a description of the 3D platform itself... PLUS a headlight!
		//  So, to add the headlight, create a new PlatformGeometry group,
		//  add the light to it, then add that platform geometry to the
		//  ViewingPlatform.
		//
		

		PlatformGeometry pg = new PlatformGeometry( );
		headlight = new DirectionalLight( );
		headlight.setColor( Env.headlightColor );
		headlight.setDirection( new Vector3f( 0.0f, 0.0f, -1.0f ) );
		headlight.setInfluencingBounds( bounds );
		headlight.setCapability( Light.ALLOW_STATE_WRITE );
		pg.addChild( headlight );
		viewingPlatform.setPlatformGeometry( pg );


		//
		//  Create the 3D content BranchGroup, containing:
		//
		//    - a TransformGroup who's transform the examine behavior
		//      will change (when enabled).
		//
		//    - 3D geometry to view
		//
		// Build the scene root
		BranchGroup sceneRoot = new BranchGroup( );

		// Build a transform that we can modify
		theSceneTransform = new TransformGroup( );
		theSceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ );
		theSceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE );
		theSceneTransform.setCapability(Group.ALLOW_CHILDREN_EXTEND );


		//
		//  Build the scene, add it to the transform, and add
		//  the transform to the scene root
		//
		Group scene = this.buildScene( );
		theSceneTransform.addChild( scene );
		sceneRoot.addChild( theSceneTransform );

		/*

		examineBehavior = new ExamineViewerBehavior(theSceneTransform, theFrame ); 
		examineBehavior.setSchedulingBounds( allBounds );
		sceneRoot.addChild( examineBehavior );

		examineBehavior.setEnable( true );
		*/

		//
		//  Compile the scene branch group and add it to the SimpleUniverse.
		//
		if ( shouldCompile ) { sceneRoot.compile( ); }
		universe.addBranchGraph( sceneRoot );
		reset( );
	}

	public void setDragResize (boolean allow) {
		theFrame.setResizable(allow);
	}

	public void setCanvasSize (int width, int height) {
		theFrame.remove(theCanvas);
		theFrame.setSize(width,height);
		theCanvas.setSize(width,height);
		theFrame.add("Center",theCanvas);
		
		topPanel.setSize(width,height);
		topPanel.remove(timeLabel);
		topPanel.add(timeLabel);
		topPanel.add(concLabel);
	}
	
	//  Reset transforms
	public void reset( )
	{
		Transform3D trans = new Transform3D( );
		trans.setScale(Env.transScale);
		theSceneTransform.setTransform( trans );
		trans.set( new Vector3f( 0.0f, 0.0f, 30.0f ) );
		theViewTransform.setTransform( trans );
	}
	
	public void rotateView() {
		double rotAng = 0.01;
		Transform3D rotT3D = new Transform3D();
		rotT3D.rotZ(rotAng);
		Transform3D curT3D = new Transform3D();
		theSceneTransform.getTransform(curT3D);
		curT3D.mul(rotT3D);
		curT3D.setScale(Env.transScale);
		theSceneTransform.setTransform(curT3D);
	}


	private BranchGroup initialBugScene() {
		bugState = new BranchGroup();
		allBranchGroupCapabilities(bugState);
		
		for (int i=0;i<Thing.thingCt;i++) {
			bugState.addChild(Thing.theThings[i].getGraphicsNode());
		}
		return bugState;
	}
	
	public static void updateBugScene() {
		Thing curThing;
		//if (bugState != null) { bugState.detach();}
		//bugState = new BranchGroup();
		//allBranchGroupCapabilities(bugState);

		for (int i=0;i<Thing.thingCt;i++) {
			curThing = Thing.theThings[i];
			if (!curThing.graphicsMade) {
				bugState.addChild(curThing.getGraphicsNode());
			} else {
				curThing.updateGraphics();
			}
		}
		//scene.addChild(bugState);
	}
	
	
	public static void restartRun () {
		synchronized(Env.safeO) {
			waitOnCanvas(theCanvas);
			Thing.removeAllThings();
			SimBounds.remakeBounds();
			Glutton.makeInitialGluttons();
			Mutton.makeInitialMuttons();
			if (!Env.remote) {
				updateBugScene();
				ratesControl.refreshAll();
			}
			Env.simulationTime = 0;
		}
	}
	
	public static void captureImage (CapturingCanvas3D theCanvas) {
		theCanvas.writeJPEG_ = true;
		theCanvas.repaint();
	}
	
	public static void waitOnCanvas (CapturingCanvas3D theCanvas) {
		while (theCanvas.isRendering()) {
			try { Thread.sleep(200); } catch (InterruptedException e) { talkln ("error sleeping"); }
		}
	}
	
	//
	//  Build scene
	//
	public Group buildScene( )
	{
		
		// Build the scene root
		scene = new Group( );
		scene.setCapability(Group.ALLOW_CHILDREN_WRITE);
		scene.setCapability(Group.ALLOW_CHILDREN_EXTEND);

		// Create application bounds
		BoundingSphere worldBounds = new BoundingSphere(
			new Point3d( 0.0, 0.0, 0.0 ),  // Center
			1000.0 );                      // Extent

		// Set the background color and its application bounds
		background = new Background( );
		background.setColor(Env.universeColor3f);
		background.setCapability( Background.ALLOW_IMAGE_WRITE );
		background.setApplicationBounds( worldBounds );
		scene.addChild( background );

		// Build foreground geometry
		scene.addChild( initialBugScene( ) );

		return scene;
	}


	public void makeTopPanel () {
		topPanel = new JPanel();
		topPanel.setSize(Env.frameWidth.getIntValue(),8);
		topPanel.setLayout(new GridLayout(1,4));
		topPanel.setBackground(Env.controlBackColor);
		
		timeLabel = new JLabel(" ");
		timeLabel.setForeground(Env.controlForeColor);
		topPanel.add(timeLabel);
		
		concLabel = new JLabel(" ");
		concLabel.setForeground(Env.controlForeColor);
		topPanel.add(concLabel);
		
		filCtLabel = new JLabel(" ");
		filCtLabel.setForeground(Env.controlForeColor);
		topPanel.add(filCtLabel);
		
		monCtLabel = new JLabel(" ");
		monCtLabel.setForeground(Env.controlForeColor);
		topPanel.add(monCtLabel);
		
	}
	
	public static String getTimeString() {
		return new String ("  Time: " + String.valueOf(timeFormat.format(Env.simulationTime)) + " s ");
	}
	
	public static String getConcString() {
		return new String (" factorX: " + String.valueOf(concFormat.format(Env.factorXConc.getValue())) + " M  "); 
	}
	
	public static String getGluttonCtString() {
		return new String (" Gluttons: " + String.valueOf(Glutton.gluttonCt));
	}
	
	public static String getMuttonCtString() {
		return new String (" Muttons: " + String.valueOf(Mutton.muttonCt));
	}
	
	public void makeMenuBar () {
		theMenuBar = new MenuBar();
		theFrame.setMenuBar( theMenuBar );

		// Add menu for program control
		Menu m = new Menu ("Program");
		m.addActionListener(this);
		
		runMenuItem = new CheckboxMenuItem ("Run",!Env.paused);
		runMenuItem.addItemListener(this);
		m.add(runMenuItem);
		
		pausedMenuItem = new CheckboxMenuItem ("Pause",Env.paused);
		pausedMenuItem.addItemListener(this);
		m.add(pausedMenuItem);
		
		m.add("Restart");
		
		m.add("--------");
		
		writeFileMenuItem = new CheckboxMenuItem("Write JPEGs");
		writeFileMenuItem.addItemListener(this);
		writeFileMenuItem.setState(Env.toFile);
		m.add(writeFileMenuItem);
		
		m.add("--------");
		
		m.add("Save Parameter Settings");
		m.add("Load Parameter Settings");
		
		theMenuBar.add(m);

		// View menu
		m = new Menu( "View" );
		m.addActionListener( this );

		m.add( "Reset view" );

		m.addSeparator( );

		headlightMenuItem = new CheckboxMenuItem( "Headlight on/off" );
		headlightMenuItem.addItemListener( this );
		headlightMenuItem.setState( headlightOnOff );
		m.add( headlightMenuItem );

		theMenuBar.add( m );
		
		// Add a menu to select among options for painting etc
		m = new Menu( "Paint_Options" );
		m.add("---General---");
		
		paintOnMenuItem = new CheckboxMenuItem("Paint On");
		paintOnMenuItem.addItemListener(this);
		paintOnMenuItem.setState(Env.paintOn);
		m.add(paintOnMenuItem);
		
		rotateMenuItem = new CheckboxMenuItem("Rotate");
		rotateMenuItem.addItemListener(this);
		rotateMenuItem.setState(Env.viewRotation);
		m.add(rotateMenuItem);
		
		m.add("---Bounding Shape---");

		appearanceMenu = new CheckboxMenuItem[2];
		appearanceMenu[0] = new CheckboxMenuItem( "Shaded" );
		appearanceMenu[0].addItemListener( this );
		appearanceMenu[0].setState( false );
		m.add( appearanceMenu[0] );

		appearanceMenu[1] = new CheckboxMenuItem( "Wireframe" );
		appearanceMenu[1].addItemListener( this );
		appearanceMenu[1].setState( true );
		m.add( appearanceMenu[1] );
		
		theMenuBar.add( m );
		
		m = new Menu ( "Panels" );
		m.addActionListener( this );
		
		m.add("Initial Conditions");
		m.add("Rates, Concentrations, Etc.");
		m.add("Graphics & File Writing");
		m.add("Render from QK files");
		
		theMenuBar.add( m );
		
		m = new Menu ( "Testing" );
		
		sphereMotionOffMenuItem = new CheckboxMenuItem("SphereThing Motion Off");
		sphereMotionOffMenuItem.addItemListener(this);
		sphereMotionOffMenuItem.setState(Env.sphereMotionOff);
		m.add(sphereMotionOffMenuItem);
		
		theMenuBar.add(m);
		
	}
	
	public void actionPerformed( ActionEvent event ) {
		String arg = event.getActionCommand( );
		
		if ( arg.equals( "Reset view" ) ) {
			reset( );
		}
	
		if (arg.equals("Restart")) { 
			setPaused();
			restartRun(); 
		}
		
		if ( arg.equals( "Initial Conditions" ) ) {
			initControl.setVisible(true);
		}
		
		if ( arg.equals( "Rates, Concentrations, Etc." ) ) {
			ratesControl.setVisible(true);
		}
		
		if ( arg.equals( "Graphics & File Writing" ) ) {
			graphicsControl.setVisible(true);
		}
		
		if ( arg.equals("Save Parameter Settings") ) {
			saveParamsToFile();
		}
		
		if ( arg.equals("Load Parameter Settings") ) {
			loadParamsFromFile();
		}
	}


	//
	//  Handle checkboxes
	//
	public void itemStateChanged( ItemEvent event )
	{
		Object src = event.getSource( );

		boolean state;
		if ( src == headlightMenuItem )
		{
			state = headlightMenuItem.getState( );
			headlight.setEnable( state );
		}
		
		// Check if it is an appearance choice
		if ( src == appearanceMenu[0] )
		{
				appearanceMenu[1].setState( false );
				SimBounds.appearanceChange(false);
				return;
		}
		if ( src == appearanceMenu[1] )
		{
				appearanceMenu[0].setState( false );
				SimBounds.appearanceChange(true);
				return;
		}
		
		if ( src == paintOnMenuItem) {
			Env.paintOn = paintOnMenuItem.getState();
		}
		
		if (src == writeFileMenuItem) {
			if (writeFileMenuItem.getState()) { 
				prepForJPEGWriting(); 
			} else {
				Env.toFile = false;
				Env.toFilePath = null;
				Env.toFileName = null;
			}
		}
		
		if (src == pausedMenuItem) {
			setPaused();	
		}
		
		if (src == runMenuItem) {
			setRunning();
		}
		
		if (src == rotateMenuItem) {
			Env.viewRotation = rotateMenuItem.getState();
		}
		
		if (src == sphereMotionOffMenuItem) {
			Env.sphereMotionOff = sphereMotionOffMenuItem.getState();
		}

	}
	
	public static void setRunning () {
		Env.paused = false;
		if (!Env.remote) {
			pausedMenuItem.setState(false);
			runMenuItem.setState(true);
		}
	}
	
	public static void setPaused () {
		Env.paused = true;
		if (!Env.remote) {
			pausedMenuItem.setState(true);
			runMenuItem.setState(false);
		}
	}
	
	public static void allBranchGroupCapabilities(BranchGroup bg) {
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		bg.setCapability(BranchGroup.ALLOW_DETACH);
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
	
	public void loadParamsFromFile () {
		String loadFromName = null;
		loadFromName = FileOps.getFileToLoad("Load parameter settings from ...", this);
		if (loadFromName == null) {
			talkln ("Parameter settings not loaded.");
		} else {
			File loadFromFile = new File(loadFromName);
			FileOps.loadParamConfig(loadFromFile);
		}
	}
	
	public void saveParamsToFile () {
		String saveToName = null;
		saveToName = FileOps.getFileToSave("Save parameter settings to ...", this);
		if (saveToName == null) {
			talkln ("Parameter settings not saved.");
		} else {
			File saveToFile = new File(saveToName);
			FileOps.writeParamConfig(saveToFile);
		}
	}
	
	public void prepForJPEGWriting () {
		Env.toFilePath = null;
		Env.toFilePath = FileOps.getDirectoryToSave("Create folder for JPEG files ...", this);
		if (Env.toFilePath == null) { 
			Env.toFile = false;
			writeFileMenuItem.setState(false);
		} else {
			int lastbitIndex = Env.toFilePath.lastIndexOf(File.separator);
			Env.toFileName = Env.toFilePath.substring(lastbitIndex+1);
			theCanvas.capturePath = Env.toFilePath;
			theCanvas.captureName = Env.toFileName;
			Env.toFile = true;
		}
	}
	

	//--------------------------------------------------------------
	//  USER INTERFACE
	//--------------------------------------------------------------


	/**
	 *  Shows the application's frame, making it and its menubar,
	 *  3D canvas, and 3D content visible.
	 */
	public void showFrame( )
	{
		theFrame.setVisible(true);
	}


	/**
	 *  Quits the application.
	 */
	public void quit( )
	{
		System.exit( 0 );
	}



	/**
	 *  Handles checkbox items on a CheckboxMenu.  The Example class has
	 *  none of its own, but subclasses may have some.
	 *
	 *  @param   menu    which CheckboxMenu needs action
	 *  @param   check   which CheckboxMenu item has changed
	 */
	public void checkboxChanged( CheckboxMenu menu, int check )
	{
		// None for us
	}


	/**
	 *  Handles a window closing event notifying the application
	 *  that the user has chosen to close the application without
	 *  selecting the "Exit" menu item.
	 *
	 *  @param   event   a WindowEvent indicating the window is closing
	 */
	public void windowClosing( WindowEvent event )
	{
		quit( );
	}
	public void windowClosed( WindowEvent event )
	{
	}
	public void windowOpened( WindowEvent event )
	{
	}
	public void windowIconified( WindowEvent event )
	{
	}
	public void windowDeiconified( WindowEvent event )
	{
	}
	public void windowActivated( WindowEvent event )
	{
	}
	public void windowDeactivated( WindowEvent event )
	{
	}



}