/**
 * Bug.java... a rod shaped bacteria with cylindrical midsection and hemispherical caps
 *
 * @author Created by Omnicore CodeGuide
 */

import java.awt.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.geometry.Cylinder;
import javax.media.j3d.*;
import javax.vecmath.*;

public class SimBounds extends Thing {
	static double length;			// length from end to end
	static double radius;			// radius of spherical caps and cylindrical section
	static double cellThickness = 0.00;		// thickness of the bacterial cell wall
	static double cylLength;
	static double halfCylLength;
	static double cylVolume;		// volume of the cylinder
	static double sphVolume;		// volume of the 2 hemispheres
	static double bugVolume;		// entire volume contained within the rod-shaped bug
	static double microMolarChangePerMonomer;	// how much to change concentration when one monomer is used or returned
	static Color bugColor = Color.white;
	
	// for graphics
	Cylinder myCyl1,myCyl2;
	ArchShape lArch,rArch;
	static Color3f ambientC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f diffuseC = new Color3f(0.9f,0.9f,0.9f);
	static Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveC = new Color3f(1.0f,1.0f,1.0f);
	static float shiny = 128.0f;
	Appearance aWire = new Appearance();
	static boolean appearanceChanged = false;
	static boolean useWireAppearance = false;
	
	// Pt3Ds reused in collision calculations
	Pt3D forceVec = new Pt3D();
	Pt3D hemDist = new Pt3D();
	Pt3D objVec = new Pt3D();
	Pt3D yzVec = new Pt3D();
	
	public SimBounds (Pt3D coord, double length, double radius) {
		super (coord);
		SimBounds.length = length;
		SimBounds.radius = radius;
		cylLength = length-2*SimBounds.radius;
		halfCylLength = cylLength/2;
		cylVolume = (length-2*radius)*Math.pow(radius,2)*Math.PI;
		sphVolume = 4*Math.PI*Math.pow(radius,3)/3;
		bugVolume = cylVolume + sphVolume;  // volume in cubic microns
		microMolarChangePerMonomer = (Math.pow(1e5,3)*1e6/(bugVolume*Env.AvogadroNum));		// (1e-6)^4 is for units.
		//talkln ("Bug contains " + String.valueOf(getParMConc()/microMolarChangePerMonomer));
	}
	
	public double getLength() {
		return length;
	}
	
	public double getRadius() {
		return radius;
	}
	
	public double getParMConc () {
		return Env.factorXConc.getValue();
	}
	
	public void takeParM (int numParM) {
		Env.factorXConc.addToValue(-numParM*microMolarChangePerMonomer);
	}
	
	public void putParM (int numParM) {
		Env.factorXConc.addToValue(numParM*microMolarChangePerMonomer);
	}
	
	public static void makeInitialBounds () {
		Thing.theSphere = new SimBounds(new Pt3D(0,0,0), Env.boundsLength.getValue(), Env.boundsRadius.getValue());
	}
	
	public static void remakeBounds () {
		Thing.theSphere.removeGraphics();
		Thing.theSphere = new SimBounds(new Pt3D(0,0,0), Env.boundsLength.getValue(), Env.boundsRadius.getValue());
	}
	
	public Pt3D rdmPtInsideBug () {
		Pt3D pt = new Pt3D();
		double fracVolInCyl = cylVolume/bugVolume;
		if (Env.mtRNG.raw() < fracVolInCyl) {  //random point in cylinder
			double yC = Env.mtRNG.raw()*2-1;
			double zC = Env.mtRNG.raw()*2-1;
			while ((yC*yC + zC+zC > 1)) { zC = Env.mtRNG.raw()*2-1; }
			pt.y = yC*radius;
			pt.z = zC*radius;
			pt.x = (Env.mtRNG.raw()*2-1)*halfCylLength;
		} else {							// random point in one of the hemispheres
			pt = Pt3D.RandomUnitVec();
			pt.scale(radius*Env.mtRNG.raw());
			if (pt.x > 0) {
				pt.x += halfCylLength;
			} else {
				pt.x += -1*halfCylLength;
			}
		}
		pt.inc(coord);
		return pt;
	}
	
	public Pt3D rdmPtInsideBug (double objRadius) {
		Pt3D pt = new Pt3D();
		double fracVolInCyl = cylVolume/bugVolume;
		if (Env.mtRNG.raw() < fracVolInCyl) {  //random point in cylinder
			double yC = Env.mtRNG.raw()*2-1;
			double zC = Env.mtRNG.raw()*2-1;
			while ((yC*yC + zC*zC > 1)) { zC = Env.mtRNG.raw()*2-1; }
			pt.y = yC*(radius-objRadius);
			pt.z = zC*(radius-objRadius);
			pt.x = (Env.mtRNG.raw()*2-1)*halfCylLength;
		} else {							// random point in one of the hemispheres
			pt = Pt3D.RandomUnitVec();
			pt.scale((radius-objRadius)*Env.mtRNG.raw());
			if (pt.x > 0) {
				pt.x += halfCylLength;
			} else {
				pt.x += -1*halfCylLength;
			}
		}
		pt.inc(coord);
		return pt;
	}
	
	public CollisionEvent amIcolliding (Pt3D ctr, double R) {
		CollisionEvent CE = null;
		objVec.sub(ctr, coord);
		yzVec.copy(0, objVec.y, objVec.z);
		double yzDist = Pt3D.vecMag(yzVec);
		if (Math.abs(objVec.x) < halfCylLength) {	// check collisions in cylindrical section of bug
			if (yzDist+R < radius) {
				return CE;
			} else {
				double impingedist = Math.abs(yzDist + R - radius);
				yzVec.unitVec();
				yzVec.reverse();  // yzVec is useful as a force vector
				CE = new CollisionEvent(impingedist, yzVec);
				return CE;
			 }
			
		} else {		// check collisions in hemispherical caps
			if (objVec.x > 0) {	// positive x hemisphere
				hemDist.copy(coord.x+halfCylLength, coord.y, coord.z);
				double topdist = Pt3D.ptDist (ctr, hemDist);
				if (topdist + R < radius) {
					return CE;
				} else {
					double impingedist = Math.abs(topdist + R - radius);
					forceVec.unitVec(topdist, hemDist, ctr);
					CE = new CollisionEvent(impingedist, forceVec);
					return CE;
				}
			}
			if (objVec.x < 0) {  // negative x hemisphere
				hemDist.copy(coord.x-halfCylLength, coord.y, coord.z);
				double botdist = Pt3D.ptDist (ctr, hemDist);
				if (botdist + R < radius) {
					return CE;
				} else {
					double impingedist = Math.abs(botdist + R - radius);
					forceVec.unitVec(botdist, hemDist, ctr);
					CE = new CollisionEvent(impingedist, forceVec);
					return CE;
				}
			}
		
		}
		return CE;
	}
	
	public static void appearanceChange(boolean useWire) {
		appearanceChanged = true;
		useWireAppearance = useWire;
	}
	
	public void setWireFrameAppearance () {
		a.setLineAttributes(new LineAttributes(2.0f, LineAttributes.PATTERN_DASH_DOT,true));
		a.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_LINE,PolygonAttributes.CULL_NONE,0.0f));
	}
	
	public void setFillAppearance () {
		a.setLineAttributes(new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID,true));
		a.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_NONE,0.0f));
	}
	
	public void makeGraphics () {
		setGraphicsCapabilities();	// set capabilities for BranchGroup, TransformGroup, and Appearance
		// the main appearance
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		ColoringAttributes cA = new ColoringAttributes(0.3f,0.3f,0.3f,ColoringAttributes.NICEST);
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(0.9f);
		tA.setTransparencyMode(TransparencyAttributes.NICEST);
		a.setMaterial(m);
		a.setColoringAttributes(cA);
		a.setTransparencyAttributes(tA);
		setWireFrameAppearance ();  // start out with the filled appearance
		
		// make alternate transparent appearance for cylinder end caps
		Appearance capA = new Appearance();
		ColoringAttributes capCA = new ColoringAttributes(0.0f,0.0f,0.0f,ColoringAttributes.NICEST);
		capA.setColoringAttributes(capCA);
		TransparencyAttributes capTA = new TransparencyAttributes ();
		capTA.setTransparency(1.0f);
		capTA.setTransparency(TransparencyAttributes.NICEST);
		capA.setTransparencyAttributes(capTA);
		capA.setLineAttributes(new LineAttributes(1.0f, LineAttributes.PATTERN_DASH_DOT,true));
		capA.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_POINT,PolygonAttributes.CULL_BACK,0.0f));
		
		//build the bug shape... first the cylindrical section.. 2 cylinders
		myCyl1 = new Cylinder((float)radius,(float)cylLength,Primitive.GENERATE_NORMALS,15,15,a);
		myCyl1.setCapability(Cylinder.ENABLE_APPEARANCE_MODIFY);
		myCyl1.setAppearance(Cylinder.TOP,capA);
		myCyl1.setAppearance(Cylinder.BOTTOM,capA);
		Transform3D cylT3D = new Transform3D();
		cylT3D.rotZ(Math.PI/2);
		TransformGroup cylTG = new TransformGroup(cylT3D);
		cylTG.addChild(myCyl1);
		
		myCyl2 = new Cylinder((float)(radius+cellThickness),(float)cylLength,Primitive.GENERATE_NORMALS,16,16,a);
		myCyl2.setCapability(Cylinder.ENABLE_APPEARANCE_MODIFY);
		myCyl2.setAppearance(Cylinder.TOP,capA);
		myCyl2.setAppearance(Cylinder.BOTTOM,capA);
		Transform3D cyl2T3D = new Transform3D();
		cyl2T3D.rotZ(Math.PI/2);
		TransformGroup cyl2TG = new TransformGroup(cyl2T3D);
		cyl2TG.addChild(myCyl2);
		
		// right hemispherical cap (arch)
		rArch = new ArchShape (0.0, Math.PI, 17, Math.PI, 0.0, 16, radius+cellThickness, radius+cellThickness, cellThickness, cellThickness, a );
		rArch.setCapability(ArchShape.ALLOW_APPEARANCE_WRITE);
		Transform3D rArchT3D = new Transform3D();
		rArchT3D.rotZ(-Math.PI/2);
		rArchT3D.setTranslation(new Vector3d(cylLength/2,0,0));
		TransformGroup rArchTG = new TransformGroup(rArchT3D);
		rArchTG.addChild(rArch);

		// left hemispherical cap (arch)
		lArch = new ArchShape (0.0, Math.PI, 17, Math.PI, 0.0, 16, radius+cellThickness, radius+cellThickness, cellThickness, cellThickness, a );
		lArch.setCapability(ArchShape.ALLOW_APPEARANCE_WRITE);
		Transform3D lArchT3D = new Transform3D();
		lArchT3D.rotZ(Math.PI/2);
		lArchT3D.setTranslation(new Vector3d(-cylLength/2,0,0));
		TransformGroup lArchTG = new TransformGroup(lArchT3D);
		lArchTG.addChild(lArch);
		
		g3d.setTransform(t3d);
		
		if (2.01*radius <= length) {	// don't draw cylindrical section if a sphere
			g3d.addChild(cylTG);
			//g3d.addChild(cyl2TG);
		}
		g3d.addChild(lArchTG);
		g3d.addChild(rArchTG);

		if (Env.showBounds) {
			G.addChild(g3d);
		}
		G.compile();
		graphicsMade = true;
	}
	public void updateGraphics () {
		if ((appearanceChanged)&(Env.showBounds)) {
				if (useWireAppearance) { setWireFrameAppearance(); } else { setFillAppearance(); }
				appearanceChanged = false;
		}
	}
}
