/**
 * SphereThing.java
 *
 * 
 */



import java.awt.*;
import com.sun.j3d.utils.geometry.Sphere;
import javax.media.j3d.*;
import javax.vecmath.*;

public class SphereThing extends Thing {
	static SphereThing [] theSpheres = new SphereThing [Env.maxThings];	// array holding all plasmids
	static int sphereCt = 0;			//
	static Object sphereThingSyncO = new Object();		// for safe removal and addition of SphereThings
	int sphereThingNumber;
	Color sphereColor = Color.green;	// color for drawing
	double radius = Env.sphereThingRadius;		// default radius

	Sphere mySphere;
	Vector3d coordVec3d = new Vector3d();
	Color3f ambientC = new Color3f(sphereColor);
	Color3f diffuseC = new Color3f(sphereColor);
	Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	Color3f emissiveC = new Color3f(0.0f,0.0f,1.0f);
	static float shiny = 128.0f;
	
	public SphereThing (Pt3D initCoord) {
		super(initCoord);
		addSphere(this);
		calculateProperties();
	}
	
	public SphereThing (Pt3D initCoord, double newRadius) {
		super(initCoord);
		addSphere(this);
		this.radius = newRadius;
		calculateProperties();
	}
	
	public void calculateProperties () {
		double radiusM = getRadius()*1.0e-6;
		bTransGam.x = 6*Math.PI*Env.aeta.getValue()*radiusM;
		bTransGam.y = bTransGam.x;
		bTransGam.z = bTransGam.x;
		bRotGam.x = 8*Math.PI*Env.aeta.getValue()*Math.pow(radiusM,3);	// drag for turning about x
		bRotGam.y = bRotGam.x;
		bRotGam.z = bRotGam.x;
		
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
	}
	
	public void grow (double aVal) { }	// to be overidden by sub-classes
	
	public void moveThing () {
		calcRandomForces();
		forceSum.inc(randForces);
		veloc.div(1.0e6, forceSum, bTransGam);
		if (!Env.sphereMotionOff) {
			coord.inc(Env.deltaT.getValue(), veloc);
		}
	}
	public void step () {
		checkBoundsCollision();		// these should add forces and torques to forceSum and torqueSum
		
		moveThing();
	
		// reset counters etc.
		resetCounters();
	}
	
	public void checkBoundsCollision () {
		CollisionEvent cE = theSphere.amIcolliding(coord,getRadius());
		if (cE == null) { return; } // no collision!
		double mag = 1.0e-6*cE.delta*bTransGam.x/Env.deltaT.getValue();
		forceSum.inc(Pt3D.Scale(mag,cE.forceUVec));
		
	}
	public static void checkSphereThingCollision () {
		SphereThing iP;
		SphereThing pP;
		for (int i=0; i<sphereCt; i++) {
			iP = theSpheres[i];
			for (int p=i+1;p<sphereCt;p++) {
				pP = theSpheres[p];
				double pDist = Pt3D.ptDist(iP.coord, pP.coord);
				if (pDist<pP.getRadius()+iP.getRadius()) {
					//double impingedist = pP.getRadius()+iP.getRadius() - pDist;
				    //Pt3D iVec = Pt3D.UnitVec(pDist, iP.coord, pP.coord);
					//Pt3D pVec = Pt3D.Reverse(iVec);
					//double mag = (1.0e-6*impingedist/Env.deltaT.getValue())/(1/iP.bTransGam.x+1/pP.bTransGam.x);
					//iP.forceSum.inc(Pt3D.Scale(mag,iVec));
					//pP.forceSum.inc(Pt3D.Scale(mag,pVec));
					
					// some eating behaviors might go here.
					if ((iP instanceof Mutton) & (pP instanceof Glutton)) {
						iP.removeMe = true;
						pP.grow(iP.radius);
						return;
					}
					if ((pP instanceof Mutton) & (iP instanceof Glutton)) {
						pP.removeMe = true;
						iP.grow(pP.radius);
						return;
					}
					
				}
			}
		}
	}
	
	public double getRadius () {
		return this.radius;
	}
	
	public void makeGraphics () {
		// make material
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		// set capabilities
		setGraphicsCapabilities();
		g3d.setTransform(t3d);
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(0.6f);
		tA.setTransparencyMode(TransparencyAttributes.NICEST);
		a.setTransparencyAttributes(tA);
		a.setMaterial(m);
		mySphere = new Sphere((float)getRadius(),Sphere.GENERATE_NORMALS, Env.plasmidTessalation, a);
		//mySphere.setCapability(Sphere.ALLOW_BOUNDS_READ);
		g3d.addChild(mySphere);
		G.addChild(g3d);
		graphicsMade = true;
	}
	
	public void remakeGraphics () {
		G.detach();
		G.removeAllChildren();
		g3d.removeAllChildren();
		graphicsMade = false;
	}

	public void updateGraphics () {
		coord.copyToVector3d(coordVec3d);
		t3d.setTranslation(coordVec3d);
		//t3d.setScale(t3d.getScale()+0.01);
		g3d.setTransform(t3d);
	 }
	
	public static void makeInitialSpheres(int numToMake) {
		for (int i=0;i<numToMake;i++) {
			makeRandomSphere(Env.sphereThingRadius);
		}
	}
	
	public static void makeRandomSphere (double rad) {
		Pt3D randomLoc = Thing.theSphere.rdmPtInsideBug(rad);
		new SphereThing (randomLoc);
	}
	
	public static void addSphere (SphereThing newSphere) {
		synchronized (sphereThingSyncO) {
			theSpheres[sphereCt] = newSphere;
			theSpheres[sphereCt].sphereThingNumber = sphereCt;
			sphereCt ++;
		}
	}
	
	public void swapOut () {
		super.swapOut();
		synchronized (sphereThingSyncO) {
			int swapId = sphereThingNumber;
			theSpheres[swapId] = theSpheres[sphereCt-1];
			theSpheres[swapId].sphereThingNumber = swapId;
			theSpheres[sphereCt-1] = null;
			sphereCt--;
		}
	}
	
	public void nullify () {
		super.nullify();
		sphereColor = null;
		mySphere = null;
		coordVec3d = null;
		ambientC = null;
		diffuseC = null;
		specularC = null;
		emissiveC = null;
	}
	
}
