/*
	// Thing.... the superclass for all moving objects in this demonstration
*/

import java.awt.*;
import javax.swing.*;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Thing extends Object {
	
	static Thing [] theThings = new Thing [Env.maxThings];		// array of all things
	static int thingCt = 0;								// how many things
	static Object syncO = new Object();	// generic object for synchronizing addition/removal of Things
	static SimBounds theSphere;					// only one bug
	// for efficiency
	static final Pt3D xUnitVector = new Pt3D(1,0,0);	// unit vector along body-fixed x-axis
	static final Pt3D yUnitVector = new Pt3D(0,1,0);	// unit vector along body-fixed y-axis
	static final Pt3D zUnitVector = new Pt3D(0,0,1);	// unit vector along body-fixed z-axis
	static final Pt3D zeroVec = new Pt3D();	// just a zero Pt3D
	int thingNumber;					// identifies where in "theThings" this Thing is
	boolean removeMe = false;			// if true this Thing will be eliminated
	Pt3D coord = new Pt3D();			// the x,y,and z position of the Thing
	Pt3D angle = new Pt3D();			//
	static Pt3D maxPos = Env.worldDimension;	// maximum x position this Thing can occupy
	double [][] transXTox = new double [3][3];	// transformation matrix from fixed to body-fixed frame
	double [][] transxToX = new double [3][3];	// the inverse transformation... body-fixed to fixed
	Pt3D uVec = new Pt3D();		// the unit vector that describes the orientation of the player
	Pt3D t1Vec = new Pt3D();		// the first transvers vector for this body... in y direction
	Pt3D t2Vec = new Pt3D();		// the second tranverse vector.. in z direction
	Pt3D cosAng = new Pt3D();	// cosines of Euler angles {psi,theta,phi}
	Pt3D sinAng = new Pt3D();	// sines of Euler angles {psi,theta,phi}
	Pt3D veloc = new Pt3D();			// the fixed frame translational velocity values Xdot, Ydot, Zdot
	Pt3D angVeloc = new Pt3D();		// the angular velocities psidot, thetadot, phidot
	Pt3D bVeloc = new Pt3D(); 		// the body-fixed frame velocities xdot, ydot, zdot
	Pt3D bAngVeloc = new Pt3D();	// the body-fixed frame angular velocities Wx, Wy, Wz
	Pt3D bTransGam = new Pt3D(); 	// body-fixed viscous translational resistances in (x,y,z}.
	Pt3D bRotGam = new Pt3D();		// body-fixed viscous rotational coefficients (Wx, Wy, Wz)
	Pt3D bTransDiff = new Pt3D();		// body-fixed translational diffusion coefficients, from bTransGam through Einstein's relation
	Pt3D bRotDiff = new Pt3D();		// body-fixed rotational diffusion coefficients
	Pt3D randForces = new Pt3D();		// random translational forces (Fx,Fy,Fz)
	Pt3D randTorques = new Pt3D();	// random rotational torques (Tx,Ty,Tz)
	Pt3D forceSum = new Pt3D(); 		// fixed-frame force sums... FX, FY, FZ
	Pt3D torqueSum = new Pt3D();		// Euler axes torque sums
	Pt3D bForceSum = new Pt3D();		// body-fixed force sum
	Pt3D bTorqueSum = new Pt3D();		// body-fixed torque sum
	
	// some Pt3Ds used in calculating random forces
	UCircRnd xVals = new UCircRnd(Env.deltaT.getValue());
	UCircRnd yVals = new UCircRnd(Env.deltaT.getValue());
	UCircRnd zVals = new UCircRnd(Env.deltaT.getValue());
	Pt3D v1 = new Pt3D();
	Pt3D v2 = new Pt3D();
	Pt3D rsq = new Pt3D();
	Pt3D facterm =new Pt3D();
	Pt3D fac1 = new Pt3D();
	Pt3D fac2 = new Pt3D();
	Pt3D tempPt = new Pt3D();
	
	// for graphics
	boolean inGroup = false;
	boolean graphicsMade = false;
	BranchGroup G = new BranchGroup();
	TransformGroup g3d = new TransformGroup();
	Transform3D t3d = new Transform3D();
	Appearance a = new Appearance();
	Material m;
	
	public Thing (Pt3D initCoord) {
		this.coord.copy(initCoord);
		addThing(this);
	}
	public Thing (Pt3D initCoord, Pt3D initAngle) {
		this.coord.copy(initCoord);
		this.angle.copy(initAngle);
		addThing(this);
	}
	public void initialize(){}
	public void calculateProperties() {}
	public void moveThing() {}
	public void step () {
		// put the code here to move this object each time-step
	}
	
	public void drawYourself (Graphics g, double scale, double [] offset) {
		// put the code here to draw the object on "g"
	}
	
	public void incForceSum (Pt3D forceToAdd) {
		forceSum.inc(forceToAdd);
	}
	
	public void incTorqueSum (Pt3D torqueToAdd) {
		torqueSum.inc(torqueToAdd);
	}
	
	public void divide() {}
	
	public void calcRandomForces () {
		// this method takes uniform deviates and finds random numbers with a
		// Gaussian distribution of mean=0, variance=2Dt, as applies for diffusive
		// motion of a particle with diffusivity D.
		// get fresh random value pairs in unit circle {v1,v2,rsq,facterm}
		xVals.newValue(Env.deltaT.getValue());
		yVals.newValue(Env.deltaT.getValue());
		zVals.newValue(Env.deltaT.getValue());
		// rearrange values into v1, v2, rsq, and facterm Pt3Ds
		v1.x = xVals.v1; v1.y = yVals.v1; v1.z = zVals.v1;
		v2.x = xVals.v2; v2.y = yVals.v2; v2.z = zVals.v2;
		rsq.x = xVals.rsq; rsq.y = yVals.rsq; rsq.z = zVals.rsq;
		facterm.x = xVals.facterm; facterm.y = yVals.facterm; facterm.z = zVals.facterm;
		// this part actually depends on the objects diffusion and drag coefficients
		tempPt.mult(bTransDiff, facterm);
		fac1.vecSqrt(tempPt);
		tempPt.mult(bRotDiff, facterm);
		fac2.vecSqrt(tempPt);
		randForces.mult(1.0/Env.deltaT.getValue(), v1, fac1, bTransGam);
		randTorques.mult(1.0/Env.deltaT.getValue(), v2, fac2, bRotGam);
	}
	public void transMat () {
		transXTox [0][0] = cosAng.x*cosAng.y;
		transXTox [0][1] = sinAng.x*cosAng.y;
		transXTox [0][2] = -sinAng.y;
		transXTox [1][0] = -sinAng.x*cosAng.z+cosAng.x*sinAng.y*sinAng.z;
		transXTox [1][1] = cosAng.x*cosAng.z+sinAng.x*sinAng.y*sinAng.z;
		transXTox [1][2] = cosAng.y*sinAng.z;
		transXTox [2][0] = sinAng.x*sinAng.z+cosAng.x*sinAng.y*cosAng.z;
		transXTox [2][1] = -cosAng.x*sinAng.z+sinAng.x*sinAng.y*cosAng.z;
		transXTox [2][2] = cosAng.y*cosAng.z;
		// the inverse transformation is found as the transpose of this matrix
		// since the direction cosine matrix is orthogonal.
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				transxToX [i][j] = transXTox [j][i];
			}
		}
	}
	public void resetCounters() {
		forceSum.zero();
		torqueSum.zero();
	}
	public double getRdmDelta (){
		return Env.mtRNG.raw()*2-1;
	}
	
	public static void addThing (Thing newThing) {
		synchronized (syncO) {
			newThing.thingNumber = thingCt;
			theThings[thingCt] = newThing;
			thingCt++;
		}
	}
	
	public void swapOut () {
		synchronized (syncO) {
			int swapId = thingNumber;
			theThings[swapId] = theThings[thingCt-1];
			theThings[swapId].thingNumber = swapId;
			theThings[thingCt-1] = null;
			thingCt--;
		}
		remove();
	}
	
	public void removeGraphics () {
		if ((!Env.remote) & (G != null)) {
			G.detach();
		}
	}
	
	public void remove () {
		removeGraphics();
		nullify();
	}
	
	public void nullify () {
		coord = null;
		angle = null;
		transXTox = null;
		transxToX = null;
		cosAng = null;
		sinAng = null;
		veloc = null;
		angVeloc = null;
		bVeloc = null;
		bAngVeloc = null;
		bTransGam = null;
		bRotGam = null;
		bTransDiff = null;
		bRotDiff = null;
		randForces = null;
		randTorques = null;
		forceSum = null;
		torqueSum = null;
		bForceSum = null;
		bTorqueSum = null;
	
		v1 = null;
		v2 = null;
		rsq = null;
		facterm = null;
		fac1 = null;
		fac2 = null;
		tempPt = null;
		
		// graphics objects
		G = null;
		g3d = null;
		t3d = null;
		a = null;
		m = null;
	}
		
	
	public static void removeDeadThings () {
		for (int i=0;i<thingCt;i++) {
			if (theThings[i] != null) {	
				if (theThings[i].removeMe) {
					theThings[i].swapOut();
				}
			}
		}
	}
	
	public static void removeAllThings () {
		for (int i=0;i<thingCt;i++) {
			if (theThings[i] != null) {	
				theThings[i].remove();
			}
		}
		thingCt = 0;
		Glutton.gluttonCt=0;
		Mutton.muttonCt=0;
		SphereThing.sphereCt=0;
	}

	public void setGraphicsCapabilities () {
		g3d.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		g3d.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		g3d.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		
		G.setCapability(BranchGroup.ALLOW_DETACH);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		G.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		
		a.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		a.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		
	}
	
	public void makeGraphics () {}
	public void updateGraphics () {}

	public Node getGraphicsNode () {
		if (!graphicsMade) { makeGraphics();}
		updateGraphics();
		return G;
	}
	public void detachGraphics () {
		G.detach();
	}
	public static void talk (String info) {
		System.out.print(info);
	}
	
	public static void talkln (String info) {
		System.out.println(info);
	}
}

